function maskString(inputString) {
    if (typeof inputString !== 'string' || inputString.length < 4) {
        return inputString;
    }
    const maskedPart = '*'.repeat(inputString.length - 4) + inputString.slice(-4);
    return maskedPart;
}

var fieldArr = ["test_consumer_key", "test_secret_key", "prod_consumer_key", "prod_secret_key"];
var displayFieldArr = ["test_display_consumer_key", "test_display_secret_key", "prod_display_consumer_key", "prod_display_secret_key"];
window.onload = function () {
    for (let i = 0; i < fieldArr.length; i++) {
        let getId = document.getElementById('woocommerce_mxmerchant_gateway_' + fieldArr[i])
        let getDisplayId = document.getElementById('woocommerce_mxmerchant_gateway_' + displayFieldArr[i]);
        if (getId && getDisplayId) {
            if (getId && getId.value) {
                getDisplayId.value = maskString(getId.value)
            }
            getDisplayId.onchange = function () {
                getId.value = getDisplayId.value
            };
            getDisplayId.onclick = function () {
                getDisplayId.value = '';
            }
        }
    }
    for (let i = 0; i < fieldArr.length; i++) {
        let getAchId = document.getElementById('woocommerce_mxmerchant_ach_gateway_' + fieldArr[i])
        let getAchDisplayId = document.getElementById('woocommerce_mxmerchant_ach_gateway_' + displayFieldArr[i]);
        if (getAchId && getAchDisplayId) {
            if (getAchId && getAchId.value) {
                getAchDisplayId.value = maskString(getAchId.value)
            }
            getAchDisplayId.onchange = function () {
                getAchId.value = getAchDisplayId.value
            };
            getAchDisplayId.onclick = function () {
                getAchDisplayId.value = '';
            }
        }
    }
    
    function setupConnection(gatewayType, buttonId, isTestMode) {
        const button = document.getElementById(buttonId);
        if (button) {
            button.classList.add('connection-button');
            button.value = isTestMode ? 'Test Sandbox Connection' : 'Test Production Connection';
            button.onclick = (e) => {
                let consumerKey
                let secretKey

                if(isTestMode && document.getElementById(`woocommerce_mxmerchant_${gatewayType}_test_display_consumer_key`).value) {
                    consumerKey = document.getElementById(`woocommerce_mxmerchant_${gatewayType}_test_consumer_key`).value
                } else if (!isTestMode && document.getElementById(`woocommerce_mxmerchant_${gatewayType}_prod_display_consumer_key`).value) {
                    consumerKey = document.getElementById(`woocommerce_mxmerchant_${gatewayType}_prod_consumer_key`).value;
                }

                if(isTestMode && document.getElementById(`woocommerce_mxmerchant_${gatewayType}_test_display_secret_key`).value) {
                    secretKey = document.getElementById(`woocommerce_mxmerchant_${gatewayType}_test_secret_key`).value
                } else if (!isTestMode && document.getElementById(`woocommerce_mxmerchant_${gatewayType}_prod_display_secret_key`).value) {
                    secretKey = document.getElementById(`woocommerce_mxmerchant_${gatewayType}_prod_secret_key`).value;
                }

                const merchantId = isTestMode
                    ? document.getElementById(`woocommerce_mxmerchant_${gatewayType}_test_merchant_id`).value
                    : document.getElementById(`woocommerce_mxmerchant_${gatewayType}_prod_merchant_id`).value;

                const callType = isTestMode ? 'sandbox.' : '';

                const API_ENDPOINT = `https://${callType}api.mxmerchant.com/checkout/v3/merchant/${merchantId}`;

                const headers = {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json',
                    'Authorization': 'Basic ' + window.btoa(consumerKey + ':' + secretKey)
                };

                fetch(API_ENDPOINT, {
                    method: 'GET',
                    headers: headers
                })
                .then((data) => {
                    if (data.ok) {
                        alert('Your connection has been successfully created.');
                    } else {
                        alert('The connection was not established.');
                    }
                })
                .catch((error) => {
                    alert('The connection was not established.');
                });
            };
        }
    }

    setupConnection('gateway', 'woocommerce_mxmerchant_gateway_check_test_card_connection', true);
    setupConnection('gateway', 'woocommerce_mxmerchant_gateway_check_card_connection', false);
    setupConnection('ach_gateway', 'woocommerce_mxmerchant_ach_gateway_check_test_card_connection', true);
    setupConnection('ach_gateway', 'woocommerce_mxmerchant_ach_gateway_check_card_connection', false);

};