<?php
/**
 * Plugin Name: WooCommerce MX Merchant Payments
 * Plugin URI: https://wordpress.org/plugins/woocommerce-gateway-mxmerchant/
 * Description: Take credit card payments on your store using MxMerchant.
 * Author: Priority
 * Author URI: https://prioritycommerce.com
 * Version: 1.7.10
 * Requires at least: 4.4
 * Tested up to: 6.342
 * WC requires at least: 2.6
 * WC tested up to: 8.4
 */

//curl -X POST -H "Authorization: Basic ZHk2WTBRTmpCVVh2UXpRTUp4a05IZ1ZvOkhPaEFHdlBFSDQrTFFxZ3dxcjZFZWhWQjVVUT0=" --request GET --url https://sandbox.api.mxmerchant.com/checkout/v3/merchant

if (!defined('ABSPATH')) {
	exit;
}

add_action('plugins_loaded', 'mxmerchant_init', 0);

function mxmerchant_init() {

	if (!class_exists('WC_Payment_Gateway'))
		return;

	/**
	 * Gateway class
	 */
	class MxMerchant_Gateway extends WC_Payment_Gateway {

		public function get_blocking_value() {
			$enabled_blocking = $this->get_option('enable_ip_blocking');
			return $enabled_blocking;
		}

		public function __construct() {
			$this->id = 'mxmerchant_gateway';
			$this->icon = apply_filters('woocommerce_mxmerchant_icon', plugins_url('assets/images/icon-new.png', __FILE__));
			$this->has_fields = false;
			$this->order_button_text = 'Process Payment';
			$this->notify_url = WC()->api_request_url('MxMerchant_Gateway');
			$this->method_title = 'MxMerchant';
			$this->method_description = 'Payment Methods Accepted: MasterCard, Visa and Discover Cards';
			$this->method_powered_by = 'Powered by MX Merchant';

			$this->init_form_fields();
			$this->init_settings();

			// Define user set variables
			$this->title = $this->get_option('title');
			$this->description = $this->get_option('description');
			$this->logo_url = $this->get_option('logo_url');
			$this->testmode = $this->get_option('testmode') === 'yes' ? true : false;
			$this->sandbox = $this->testmode ? 'sandbox.' : '';
			// $this->merchant_id = $this->get_option('merchant_id');

			$this->public_test_key = $this->get_option('test_consumer_key');
			$this->private_test_key = $this->get_option('test_secret_key');

			$this->public_live_key = $this->get_option('prod_consumer_key');
			$this->private_live_key = $this->get_option('prod_secret_key');
			if(!($this->testmode) && empty($this->get_option('prod_merchant_id'))) {
				$this->update_option('prod_merchant_id', $this->get_option('merchant_id'));
			}
			if($this->testmode && empty($this->get_option('test_merchant_id'))) {
				$this->update_option('test_merchant_id', $this->get_option('merchant_id'));
			}
			$this->merchant_id = $this->testmode ? $this->get_option('test_merchant_id') : $this->get_option('prod_merchant_id');

			$this->public_key = $this->testmode ? $this->public_test_key : $this->public_live_key;
			$this->private_key = $this->testmode ? $this->private_test_key : $this->private_live_key;
			wp_register_style('mxm_css', plugins_url( 'assets/css/mxm.css', __FILE__ ), array(), '1.7.3' );
			wp_enqueue_style('mxm_css');
			//Actions
			add_action('wp_enqueue_scripts', array($this, 'payment_scripts'));
			add_action('woocommerce_receipt_mxmerchant_gateway', array($this, 'receipt_page'));
			add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
			// Payment listener/API hook
			add_action('woocommerce_api_wc_mxmerchant_gateway', array($this, 'charge_token'));

			// Check if the gateway can be used
			if ( ! $this->is_valid_for_use()) {
				$this->enabled = false;
			}
		}

		/**
		 * Check if the store curreny is set to NGN
		 * */
		public function is_valid_for_use() {
			return true;
		}

		/**
		 * Check if this gateway is enabled
		 */
		public function is_available() {
			if ($this->enabled == "yes") {
				if (!( $this->public_key && $this->private_key )) {
					return false;
				}
				return true;
			}
			return false;
		}

		/**
		 * Admin Panel Options
		 * */
		public function admin_options() {
			echo '<h3>MxMerchant</h3>';
			echo '<p>MxMerchant WooCommerce Payment Gateway allows you to accept payment on your WooCommerce store via the MxMerchant Developer API.</p>';
			echo '<p>To open a MxMerchant account click <a href="https://mxmerchant.com/" target="_blank">here</a>';

			if ($this->is_valid_for_use()) {

				echo '<table class="form-table">';
				$this->generate_settings_html();
				echo '</table>';
			} else {
			?>
				<div class="inline error"><p><strong>MxMerchant Payment Gateway Disabled</strong>: <?php echo $this->msg ?></p></div>
			<?php
			}
		}

		/**
		 * Initialise Gateway Settings Form Fields
		 * */
		function init_form_fields() {

			$this->form_fields = array(
				'enabled' => array(
					'title' => 'Enable/Disable',
					'type' => 'checkbox',
					'label' => 'Enable MxMerchant Payment Gateway',
					'description' => 'Enable or disable the gateway.',
					'desc_tip' => true,
					'default' => 'yes'
				),
				'title' => array(
					'title' => 'Title',
					'type' => 'text',
					'description' => 'This controls the title which the user sees during checkout.',
					'desc_tip' => false,
					'default' => 'Credit Card'
				),
				'description' => array(
					'title' => 'Description',
					'type' => 'textarea',
					'description' => 'This controls the description which the user sees during checkout.',
					'default' => 'Payment Methods Accepted: MasterCard, Visa, Discover'
				),
				'test_merchant_id' => array(
					'title' => 'Sandbox Merchant ID',
					'type' => 'text',
					'description' => 'Enter your Sandbox Merchant ID.',
					'default' => ''
				),
				'test_display_consumer_key' => array(
					'title' => 'Sandbox Consumer Key',
					'type' => 'text',
					'description' => 'Enter your Sandbox Consumer Key here.',
					'default' => ''
				),
				'test_display_secret_key' => array(
					'title' => 'Sandbox Secret Key',
					'type' => 'text',
					'description' => 'Enter your Sandbox Secret Key here.',
					'default' => ''
				),
				'check_test_card_connection' => array(
					'title' => 'Check Sandbox Connection',
					'type' => 'button',
					'label' => 'Check Sandbox Connection',
					'default' => 'Test Sandbox Connection',
					'description' => 'Click the button to verify the Sandbox connection.', 
				),
				'prod_merchant_id' => array(
					'title' => 'Production Merchant ID',
					'type' => 'text',
					'description' => 'Enter your Production Merchant ID.',
					'default' => ''
				),
				'prod_display_consumer_key' => array(
					'title' => 'Production Consumer Key',
					'type' => 'text',
					'description' => 'Enter your Production Consumer Key here.',
					'default' => ''
				),
				'prod_display_secret_key' => array(
					'title' => 'Production Secret Key',
					'type' => 'text',
					'description' => 'Enter your Production Secret Key here.',
					'default' => ''
				),
				'check_card_connection' => array(
					'title' => 'Check Production Connection',
					'type' => 'button',
					'label' => 'Check Production Connection',
					'default' => 'Test Production Connection',
					'description' => 'Click the button to verify the Production connection.', 
				),
				'testing' => array(
					'title' => 'Gateway Testing',
					'type' => 'title',
					'description' => '',
				),
				'testmode' => array(
					'title' => 'Test Mode',
					'type' => 'checkbox',
					'label' => 'Enable Test Mode',
					'default' => 'no',
					'description' => 'Test mode enables you to test payments before going live. <br />If you ready to start receiving payment on your site, kindly uncheck this.',
				),
				'custom_error_message' => array(
					'title' => 'Add Error Message',
					'type' => 'text',
					'label' => 'Add Error Message',
					'default' => 'Your IP is blocked from placing an order. Please contact customer support for further assistance.',
					'description' => 'Enter the error message for the user.', 
				),
				'enable_ip_blocking' => array(
					'title' => 'Enable IP Block',
					'type' => 'checkbox',
					'label' => 'Enable IP Blocking',
					'default' => 'yes',
					'description' => 'Check the box to enable IP blocking.', 
				),
				'test_consumer_key' => array(
					'type' => 'hidden',
				),
				'test_secret_key' => array(
					'type' => 'hidden',
				),
				'prod_consumer_key' => array(
					'type' => 'hidden',
				),
				'prod_secret_key' => array(
					'type' => 'hidden',
				),
			);
		}

		

			/* Start of credit card form */
			public function payment_fields() {
				if(isset($this->authorizenet_description)) {
					echo apply_filters('wc_authorizenet_description', wpautop(wp_kses_post(wptexturize(trim($this->authorizenet_description)))));
				}
				$this->form();
			}

			public function field_name($name) {
				return $this->supports('tokenization') ? '' : ' name="' . esc_attr($this->id . '-' . $name) . '" ';
			}

			public function form() {
				wp_enqueue_script('wc-credit-card-form');
				$fields = array();
				$cvc_field = '<p class="form-row form-row-last">
			<label for="' . esc_attr($this->id) . '-card-cvc">' . __('Card Code', 'woocommerce') . ' <span class="required">*</span></label>
			<input id="' . esc_attr($this->id) . '-card-cvc" class="input-text wc-credit-card-form-card-cvc" type="text" autocomplete="off" placeholder="' . esc_attr__('CVC', 'woocommerce') . '" ' . $this->field_name('card-cvc') . '/>
		</p>';
				$default_fields = array(
					'card-number-field' => '<p class="form-row form-row-wide">
				<label for="' . esc_attr($this->id) . '-card-number">' . __('Card Number', 'woocommerce') . ' <span class="required">*</span></label>
				<input id="' . esc_attr($this->id) . '-card-number" class="input-text wc-credit-card-form-card-number" type="text" maxlength="20" autocomplete="off" placeholder="&bull;&bull;&bull;&bull; &bull;&bull;&bull;&bull; &bull;&bull;&bull;&bull; &bull;&bull;&bull;&bull;" ' . $this->field_name('card-number') . ' />
			</p>',
					'card-expiry-field' => '<p class="form-row form-row-first">
				<label for="' . esc_attr($this->id) . '-card-expiry">' . __('Expiry (MM/YY)', 'woocommerce') . ' <span class="required">*</span></label>
				<input id="' . esc_attr($this->id) . '-card-expiry" class="input-text wc-credit-card-form-card-expiry" type="text" autocomplete="off" placeholder="' . esc_attr__('MM / YY', 'woocommerce') . '" ' . $this->field_name('card-expiry') . ' />
			</p>',
					'card-cvc-field' => $cvc_field
				);

				$fields = wp_parse_args($fields, apply_filters('woocommerce_credit_card_form_fields', $default_fields, $this->id));
				?>

				<fieldset id="wc-<?php echo esc_attr($this->id); ?>-cc-form" class='wc-credit-card-form wc-payment-form'>
				<?php do_action('woocommerce_credit_card_form_start', $this->id); ?>
				<?php
				foreach ($fields as $field) {
					echo $field;
				}
				?>
				<?php do_action('woocommerce_credit_card_form_end', $this->id); ?>
					<div class="clear"></div>
				</fieldset>
				<p class="powered_by_text"><?php echo $this->method_powered_by; ?></p>
				<?php
			}
		
		/**
		 * Outputs scripts used for MxMerchant payment
		 */
		public function payment_scripts() {

			if (!is_checkout_pay_page()) {
				return;
			}

			$suffix = ( defined('SCRIPT_DEBUG') && SCRIPT_DEBUG ) ? '' : '.min';

			wp_enqueue_script('tbz_mxmerchant', 'https://checkout.simplepay.ng/v2/simplepay.js', array('jquery'), '1.0.0', true);

			wp_enqueue_script('mxmerchant', plugins_url('assets/js/simplepay' . $suffix . '.js', __FILE__), array('tbz_mxmerchant'), '1.0.0', true);

			if (is_checkout_pay_page() && get_query_var('order-pay')) {

				$order_key = urldecode($_GET['key']);
				$order_id = absint(get_query_var('order-pay'));

				$order = wc_get_order($order_id);

				$email = method_exists($order, 'get_billing_email') ? $order->get_billing_email() : $order->billing_email;
				$billing_address_1 = method_exists($order, 'get_billing_address_1') ? $order->get_billing_address_1() : $order->billing_address_1;
				$billing_address_2 = method_exists($order, 'get_billing_address_2') ? $order->get_billing_address_2() : $order->billing_address_2;
				$city = method_exists($order, 'get_billing_city') ? $order->get_billing_city() : $order->billing_city;
				$country = method_exists($order, 'get_billing_country') ? $order->get_billing_country() : $order->billing_country;

				$amount = $order->get_total() * 100;
				$address = $billing_address_1 . ' ' . $billing_address_2;

				$description = 'Payment for Order #' . $order_id;

				$the_order_id = method_exists($order, 'get_id') ? $order->get_id() : $order->id;
				$the_order_key = method_exists($order, 'get_order_key') ? $order->get_order_key() : $order->order_key;

				if ($the_order_id == $order_id && $the_order_key == $order_key) {
					$simplepay_params['key'] = $this->public_key;
					$simplepay_params['email'] = $email;
					$simplepay_params['address'] = $address;
					$simplepay_params['city'] = $city;
					$simplepay_params['country'] = $country;
					$simplepay_params['amount'] = $amount;
					$simplepay_params['order_id'] = $order_id;
					$simplepay_params['description'] = $description;
					$simplepay_params['currency'] = 'NGN';
					$simplepay_params['logo'] = $this->logo_url;
				}
			}

			wp_localize_script('mxmerchant', 'mxmerchant_params', $simplepay_params);
		}
		
		function add_error_ip_address($ip) {
			global $wpdb, $table_prefix;
			$wp_block = $table_prefix.'payment_request';
			$q = $wpdb->prepare("SELECT * FROM $wp_block WHERE requestIP = %s", $ip);
			$results = $wpdb->get_row($q);
			if(!empty($results)) {
				$updated_attempts = $results->attempt + 1;
				if($updated_attempts == 5) {
					$q = $wpdb->prepare("UPDATE $wp_block SET `attempt` = %d , `status` =  %s  WHERE requestIP = %s", $updated_attempts, "BLOCK", $ip);
					$wpdb->get_row($q);
				} else {
					$q = $wpdb->prepare("UPDATE $wp_block SET `attempt` = %d WHERE requestIP = %s", $updated_attempts, $ip);
					$wpdb->get_row($q);
				}
			} else {
				$data = array(
					'requestIP' => $ip,
					'time' => date("y-m-d h:i:s"),
					'attempt' => 1,
					'status' => 'OPEN'		
				);
				$wpdb->insert($wp_block, $data);
			}
		}

		function check_ip_address_before_payment($ip, $wc_order) {
			global $wpdb, $table_prefix;
			$wp_block = $table_prefix.'payment_request';
			$q = $wpdb->prepare("SELECT * FROM $wp_block WHERE requestIP = %s", $ip);
			$results = $wpdb->get_row($q);
			if(!empty($results)) {
				$hour_diff = round((strtotime(date("y-m-d h:i:s")) - strtotime($results->time))/3600, 1);
				if($hour_diff > 12) {
					$q = $wpdb->prepare("DELETE FROM $wp_block WHERE requestIP = %s", $ip);
					$wpdb->get_row($q);
				}
			}

			$q = $wpdb->prepare("SELECT * FROM $wp_block WHERE requestIP = %s", $ip);
			$results = $wpdb->get_row($q);
			if(!empty($results)) {
				if($results->status == 'BLOCK') {
					return "BLOCK";
				} 
			}
			return "ALLOW";
		}

		/**
		 * Process the payment and return the result
		 * */
		public function process_payment($order_id) {
			$enabled_blocking = $this->get_option('enable_ip_blocking');
			$error_message = $this->get_option('custom_error_message');
			$wc_order = wc_get_order($order_id);
			$check_block = $this->check_ip_address_before_payment($wc_order->customer_ip_address, $wc_order);
			if($check_block == 'BLOCK' && $enabled_blocking == 'yes') {
				$wc_order->add_order_note(__($error_message.', error on ' . date("d-M-Y h:i:s e"), 'woocommerce'));	 
				if($this->is_checkout_block()){
					$message = __( $error_message, 'woocommerce' );
					throw new Exception( $message );
				}else{
					wc_add_notice($error_message, 'error');
				}
			} else {
				$card_num = sanitize_text_field(str_replace(' ', '', $_POST['mxmerchant_gateway-card-number']));
				$exp_date = explode('/', sanitize_text_field($_POST['mxmerchant_gateway-card-expiry']));
				$exp_month = str_replace(' ', '', $exp_date[0]);
				$exp_year = str_replace(' ', '', $exp_date[1]);

				if (strlen($exp_year) == 2) {
					$exp_year += 2000;
				}
				
				$cvc = sanitize_text_field($_POST['mxmerchant_gateway-card-cvc']);

				$data = [
					'merchantId' => $this->merchant_id,
					'amount' => $wc_order->get_total(),
					'tax' => $wc_order->get_total_tax(),
					'shipAmount' => $wc_order->get_total_shipping(),
					'paymentType' => 'Sale',
					'tenderType' => 'Card',
					'cardAccount' => [
						'number' => $card_num,
						'expiryMonth' => $exp_month,
						'expiryYear' => $exp_year,
						'avsStreet' => $wc_order->get_billing_address_1(),
						'cvv' => $cvc
					],
					'customerName' => $wc_order->get_billing_first_name() . ' ' . $wc_order->get_billing_last_name(),
					'source' => 'API - WooCommerce',
					'meta' => get_bloginfo('blogname') . ' Order #'.$wc_order->get_order_number()
				];
	
				// Validate Format : XXXXX-XXXX OR XXXXX OR XXXXXXXXX OR X#X#X# OR X#X #X#
				$postal_code = $wc_order->get_billing_postcode();
				if(preg_match("/^((\d{5}-\d{4})|(\d{5})|(\d{9})|(?<full>(?<part1>[ABCEGHJKLMNPRSTVXY]{1}\d{1}[A-Z]{1})(?:[ ](?=\d))?(?<part2>\d{1}[A-Z]{1}\d{1})))$/", $postal_code)){
				// if(preg_match("/^\d{5}-\d{4}$/", $postal_code) || preg_match("/^[0-9]{5,5}$/", $postal_code) || preg_match("/^[0-9]{9,9}$/", $postal_code)){
					$data['cardAccount']['avsZip'] = $postal_code;
				}
	
				$result = $this->_api('checkout/v3/payment', $data);
				$json = json_decode($result);
				if ($json) {
					if ($json->status && $json->reference && $json->status === 'Approved') {
						$wc_order->add_order_note(__( $json->authMessage. ' on '.date("d-M-Y h:i:s e"). ' with Transaction ID = ' . $json->reference . ' using ' . strtoupper($json->tenderType) . ' and authorization code ' . $json->authCode, 'woocommerce'));
						$wc_order->payment_complete($json->reference);
						WC()->cart->empty_cart();
						return array (
							'result'   => 'success',
							'redirect' => $this->get_return_url( $wc_order ),
						);
					} else if ($json->errorCode && $json->details && isset($json->details[0])) {
						$this->add_error_ip_address($wc_order->customer_ip_address);
						$wc_order->add_order_note(__($json->details[0] . ' on ' . date("d-M-Y h:i:s e"), 'woocommerce'));	 
						if($this->is_checkout_block()){
							$message = __( implode( ',<br/>', $json->details ), 'woocommerce' );
							throw new Exception( $message );
						}else{
							wc_add_notice(implode( ',<br/>', $json->details ), 'error');
						}
					} else {
						$this->add_error_ip_address($wc_order->customer_ip_address);
						$wc_order->add_order_note(__('Unknown error on ' . date("d-M-Y h:i:s e"), 'woocommerce'));							
						if($this->is_checkout_block()){
							$message = __( 'Unknown error', 'woocommerce' );
							throw new Exception( $message );
						}else{
							wc_add_notice('Unknown error', 'error');
						}
					}
				} else if ($json->status) {
					$this->add_error_ip_address($wc_order->customer_ip_address);
					$wc_order->add_order_note(__($json->status . ' on ' . date("d-M-Y h:i:s e"), 'woocommerce'));	 
					if($this->is_checkout_block()){
						$message = __( $json->status, 'woocommerce' );
						throw new Exception( $message );
					}else{
						wc_add_notice($json->status, 'error');
					}
				} else {
					$this->add_error_ip_address($wc_order->customer_ip_address);
					$wc_order->add_order_note(__('Unknown error on ' . date("d-M-Y h:i:s e"), 'woocommerce'));	 
					if($this->is_checkout_block()){
						$message = __( 'Unknown error', 'woocommerce' );
						throw new Exception( $message );
					}else{
						wc_add_notice('Unknown error', 'error');
					}
				}
			}
		}

		function is_checkout_block()
		{
			$post = get_post(get_option('woocommerce_checkout_page_id'));
			return false !== strpos($post->post_content, '<!-- wp:woocommerce/checkout');
		}

		/**
		 * Output for the order received page.
		 * */
		public function receipt_page($order_id) {

			$order = wc_get_order($order_id);

			echo '<p>Thank you for your order</p>';
 
		}
		
		private function _api($endpoint, $data) {
			$ch = curl_init('https://' . $this->sandbox . 'api.mxmerchant.com/' . $endpoint . '?echo=true');

			$data_string = json_encode($data);
			
			$header = array(
				'Accept: application/json',
				'Content-Type: application/json',
				'Content-Length: ' . strlen($data_string),
				'Authorization: Basic ' . base64_encode($this->public_key . ':' . $this->private_key)
			);
			
			curl_setopt($ch, CURLOPT_CUSTOMREQUEST, "POST");                                                                     
			curl_setopt($ch, CURLOPT_POSTFIELDS, $data_string);                                                                  
			curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);                                                                      
			curl_setopt($ch, CURLOPT_HTTPHEADER, $header);                                                                                                                   
			curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);                                                                                                                   

			return curl_exec($ch);
		}

		/**
		 * Verify a payment token
		 * */
		public function charge_token() {

			
			if (isset($_POST['mxmerchant_token'], $_POST['mxmerchant_order_id'])) {

				$verify_url = 'https://checkout.simplepay.ng/v2/payments/card/charge/';

				$order_id = (int) $_POST['mxmerchant_order_id'];

				$order = wc_get_order($order_id);
				$order_total = $order->get_total() * 100;

				$headers = array(
					'Content-Type' => 'application/json',
					'Authorization' => 'Basic ' . base64_encode($this->private_key . ':' . '')
				);

				$body = array(
					'token' => $_POST['mxmerchant_token'],
					'amount' => $order_total,
					'amount_currency' => 'NGN',
				);

				$args = array(
					'headers' => $headers,
					'body' => json_encode($body),
					'timeout' => 60,
					'method' => 'POST'
				);

				$request = wp_remote_post($verify_url, $args);

				if (!is_wp_error($request) && 200 == wp_remote_retrieve_response_code($request)) {

					$simplepay_response = json_decode(wp_remote_retrieve_body($request));

					$amount_paid = $simplepay_response->amount;
					$transaction_id = $simplepay_response->id;

					do_action('mxmerchant_after_payment', $simplepay_response);

					if ('20000' == $simplepay_response->response_code) {

						// check if the amount paid is equal to the order amount.
						if ($amount_paid < $order_total) {

							//Update the order status
							$order->update_status('on-hold', '');

							add_post_meta($order_id, '_transaction_id', $transaction_id, true);

							//Error Note
							$notice = 'Thank you for shopping with us.<br />The payment was successful, but the amount paid is not the same as the order amount.<br />Your order is currently on-hold.<br />Kindly contact us for more information regarding your order and payment status.';

							$notice_type = 'notice';

							//Add Admin Order Note
							$order->add_order_note('Look into this order. <br />This order is currently on hold.<br />Reason: Amount paid is less than the order amount.<br />Amount Paid was &#8358;' . $amount_paid / 100 . ' while the order amount is &#8358;' . $order_total / 100 . '<br />Simplepay Transaction ID: ' . $transaction_id);

							// Reduce stock levels
							$order->reduce_order_stock();

							wc_add_notice($notice, $notice_type);
						} else {

							$order->payment_complete($transaction_id);

							$order->add_order_note(sprintf('Payment via Simplepay successful (Transaction ID: %s)', $transaction_id));
						}

						wc_empty_cart();

						wp_redirect($this->get_return_url($order));

						exit;
					} else {

						wp_redirect(wc_get_page_permalink('checkout'));

						exit;
					}
				}
			}

			wp_redirect(wc_get_page_permalink('checkout'));

			exit;
		}

	}

	/**
	 * ACH Gateway class
	 */
	class MxMerchant_ACH_Gateway extends WC_Payment_Gateway {

		public function get_ach_blocking_value() {
			global $enabled_ach_blocking; 
			$enabled_ach_blocking = $this->get_option('enable_ip_blocking');
			return $enabled_ach_blocking;
		}

		public function __construct() {
			$this->id = 'mxmerchant_ach_gateway';
			$this->icon = apply_filters('woocommerce_mxmerchant_icon', plugins_url('assets/images/icon.png', __FILE__));
			$this->has_fields = false;
			$this->order_button_text = 'Process Payment';
			$this->notify_url = WC()->api_request_url('MxMerchant_ACH_Gateway');
			$this->method_title = 'MxMerchant ACH';
			$this->method_description = 'Payment Methods Accepted: MasterCard, Visa and Discover Cards';

			$this->init_form_fields();
			$this->init_settings();

			// Define user set variables
			$this->title = $this->get_option('title');
			$this->description = $this->get_option('description');
			$this->logo_url = $this->get_option('logo_url');
			$this->testmode = $this->get_option('testmode') === 'yes' ? true : false;

			$this->public_test_key = $this->get_option('test_consumer_key');
			$this->private_test_key = $this->get_option('test_secret_key');

			$this->public_live_key = $this->get_option('prod_consumer_key');
			$this->private_live_key = $this->get_option('prod_secret_key');
			$this->merchant_id = $this->testmode ? $this->get_option('test_merchant_id') : $this->get_option('prod_merchant_id');

			$this->public_key = $this->testmode ? $this->public_test_key : $this->public_live_key;
			$this->private_key = $this->testmode ? $this->private_test_key : $this->private_live_key;
			wp_register_style('mxm_css', plugins_url( 'assets/css/mxm.css', __FILE__ ), array(), '1.7.3' );
			wp_enqueue_style('mxm_css');
			//Actions
			add_action('wp_enqueue_scripts', array($this, 'payment_scripts'));
			add_action('woocommerce_receipt_mxmerchant_gateway', array($this, 'receipt_page'));
			add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
			// Payment listener/API hook
			add_action('woocommerce_api_wc_mxmerchant_gateway', array($this, 'charge_token'));

			// Check if the gateway can be used
			if ( ! $this->is_valid_for_use()) {
				$this->enabled = false;
			}
		}

		/**
		 * Check if the store curreny is set to NGN
		 * */
		public function is_valid_for_use() {
			return true;
		}

		/**
		 * Check if this gateway is enabled
		 */
		public function is_available() {
			if ($this->enabled == "yes") {
				if (!( $this->public_key && $this->private_key )) {
					return false;
				}
				return true;
			}
			return false;
		}

		/**
		 * Admin Panel Options
		 * */
		public function admin_options() {
			echo '<h3>MxMerchant</h3>';
			echo '<p>MxMerchant WooCommerce Payment Gateway allows you to accept payment on your WooCommerce store via the MxMerchant Developer API.</p>';
			echo '<p>To open a MxMerchant account click <a href="https://mxmerchant.com/" target="_blank">here</a>';

			if ($this->is_valid_for_use()) {

				echo '<table class="form-table">';
				$this->generate_settings_html();
				echo '</table>';
			} else {
				?>

				<div class="inline error"><p><strong>MxMerchant Payment Gateway Disabled</strong>: <?php echo $this->msg ?></p></div>

				<?php
			}
		}

		/**
		 * Initialise Gateway Settings Form Fields
		 * */
		function init_form_fields() {

			$this->form_fields = array(
				'enabled' => array(
					'title' => 'Enable/Disable',
					'type' => 'checkbox',
					'label' => 'Enable MxMerchant Payment Gateway',
					'description' => 'Enable or disable the gateway.',
					'desc_tip' => true,
					'default' => 'yes'
				),
				'title' => array(
					'title' => 'Title',
					'type' => 'text',
					'description' => 'This controls the title which the user sees during checkout.',
					'desc_tip' => false,
					'default' => 'Online Check'
				),
				'description' => array(
					'title' => 'Description',
					'type' => 'textarea',
					'description' => 'This controls the description which the user sees during checkout.',
					'default' => 'Payment Methods Accepted: MasterCard, Visa, Discover'
				),
				'test_merchant_id' => array(
					'title' => 'Sandbox Merchant ID',
					'type' => 'text',
					'description' => 'Enter your Sandbox Merchant ID.',
					'default' => ''
				),
				'test_display_consumer_key' => array(
					'title' => 'Sandbox Consumer Key',
					'type' => 'text',
					'description' => 'Enter your Sandbox Consumer Key here.',
					'default' => ''
				),
				'test_display_secret_key' => array(
					'title' => 'Sandbox Secret Key',
					'type' => 'text',
					'description' => 'Enter your Sandbox Secret Key here.',
					'default' => ''
				),
				'check_test_card_connection' => array(
					'title' => 'Check Sandbox Connection',
					'type' => 'button',
					'label' => 'Check Sandbox Connection',
					'default' => 'Test Sandbox Connection',
					'description' => 'Click the button to verify the Sandbox connection.', 
				),
				'prod_merchant_id' => array(
					'title' => 'Production Merchant ID',
					'type' => 'text',
					'description' => 'Enter your Production Merchant ID.',
					'default' => ''
				),
				'prod_display_consumer_key' => array(
					'title' => 'Production Consumer Key',
					'type' => 'text',
					'description' => 'Enter your Production Consumer Key here.',
					'default' => ''
				),
				'prod_display_secret_key' => array(
					'title' => 'Production Secret Key',
					'type' => 'text',
					'description' => 'Enter your Production Secret Key here.',
					'default' => ''
				),
				'check_card_connection' => array(
					'title' => 'Check Production Connection',
					'type' => 'button',
					'label' => 'Check Production Connection',
					'default' => 'Test Production Connection',
					'description' => 'Click the button to verify the Production connection.', 
				),
				'testing' => array(
					'title' => 'Gateway Testing',
					'type' => 'title',
					'description' => '',
				),
				'testmode' => array(
					'title' => 'Test Mode',
					'type' => 'checkbox',
					'label' => 'Enable Test Mode',
					'default' => 'no',
					'description' => 'Test mode enables you to test payments before going live. <br />If you ready to start receiving payment on your site, kindly uncheck this.',
				),
				'custom_error_message' => array(
					'title' => 'Add Error Message',
					'type' => 'text',
					'label' => 'Add Error Message',
					'default' => 'Your IP is blocked from placing an order. Please contact customer support for further assistance.',
					'description' => 'Enter the error message for the user.', 
				),
				'enable_ip_blocking' => array(
					'title' => 'Enable IP Block',
					'type' => 'checkbox',
					'label' => 'Enable IP Blocking',
					'default' => 'yes',
					'description' => 'Check the box to enable IP blocking.', 
				),
				'test_consumer_key' => array(
					'type' => 'hidden',
				),
				'test_secret_key' => array(
					'type' => 'hidden',
				),
				'prod_consumer_key' => array(
					'type' => 'hidden',
				),
				'prod_secret_key' => array(
					'type' => 'hidden',
				),
			);
		}

		

			/**
	 * Builds our payment fields area - including tokenization fields for logged
	 * in users, and the actual payment fields.
	 * @since 2.6.0
	 */
	public function payment_fields() {
		if ( $this->supports( 'tokenization' ) && is_checkout() ) {
			$this->tokenization_script();
			$this->saved_payment_methods();
			$this->form();
			$this->save_payment_method_checkbox();
		} else {
			$this->form();
		}
	}

	/**
	 * Outputs fields for entering eCheck information.
	 * @since 2.6.0
	 */
	public function form() {
		$fields = array();

		$default_fields = array(
			'routing-number' => '<p class="form-row form-row-first">
				<label for="' . esc_attr( $this->id ) . '-routing-number">' . esc_html__( 'Routing number', 'woocommerce' ) . ' <span class="required">*</span></label>
				<input id="' . esc_attr( $this->id ) . '-routing-number" class="input-text wc-echeck-form-routing-number" type="text" maxlength="9" autocomplete="off" placeholder="&bull;&bull;&bull;&bull;&bull;&bull;&bull;&bull;&bull;" name="' . esc_attr( $this->id ) . '-routing-number" />
			</p>',
			'account-number' => '<p class="form-row form-row-wide">
				<label for="' . esc_attr( $this->id ) . '-account-number">' . esc_html__( 'Account number', 'woocommerce' ) . ' <span class="required">*</span></label>
				<input id="' . esc_attr( $this->id ) . '-account-number" class="input-text wc-echeck-form-account-number" type="text" autocomplete="off" name="' . esc_attr( $this->id ) . '-account-number" maxlength="17" />
			</p>',
		);

		$fields = wp_parse_args( $fields, apply_filters( 'woocommerce_echeck_form_fields', $default_fields, $this->id ) );
		?>

		<fieldset id="<?php echo esc_attr( $this->id ); ?>-cc-form" class='wc-echeck-form wc-payment-form'>
			<?php do_action( 'woocommerce_echeck_form_start', $this->id ); ?>
			<?php
				foreach ( $fields as $field ) {
					echo $field;
				}
			?>
			<?php do_action( 'woocommerce_echeck_form_end', $this->id ); ?>
			<div class="clear"></div>
		</fieldset><?php
	}

			public function field_name($name) {
				return $this->supports('tokenization') ? '' : ' name="' . esc_attr($this->id . '-' . $name) . '" ';
			}

		
		/**
		 * Outputs scripts used for MxMerchant payment
		 */
		public function payment_scripts() {

			if (!is_checkout_pay_page()) {
				return;
			}

			$suffix = ( defined('SCRIPT_DEBUG') && SCRIPT_DEBUG ) ? '' : '.min';

			wp_enqueue_script('tbz_mxmerchant', 'https://checkout.simplepay.ng/v2/simplepay.js', array('jquery'), '1.0.0', true);

			wp_enqueue_script('mxmerchant', plugins_url('assets/js/simplepay' . $suffix . '.js', __FILE__), array('tbz_mxmerchant'), '1.0.0', true);

			if (is_checkout_pay_page() && get_query_var('order-pay')) {

				$order_key = urldecode($_GET['key']);
				$order_id = absint(get_query_var('order-pay'));

				$order = wc_get_order($order_id);

				$email = method_exists($order, 'get_billing_email') ? $order->get_billing_email() : $order->billing_email;
				$billing_address_1 = method_exists($order, 'get_billing_address_1') ? $order->get_billing_address_1() : $order->billing_address_1;
				$billing_address_2 = method_exists($order, 'get_billing_address_2') ? $order->get_billing_address_2() : $order->billing_address_2;
				$city = method_exists($order, 'get_billing_city') ? $order->get_billing_city() : $order->billing_city;
				$country = method_exists($order, 'get_billing_country') ? $order->get_billing_country() : $order->billing_country;

				$amount = $order->get_total() * 100;
				$address = $billing_address_1 . ' ' . $billing_address_2;

				$description = 'Payment for Order #' . $order_id;

				$the_order_id = method_exists($order, 'get_id') ? $order->get_id() : $order->id;
				$the_order_key = method_exists($order, 'get_order_key') ? $order->get_order_key() : $order->order_key;

				if ($the_order_id == $order_id && $the_order_key == $order_key) {
					$simplepay_params['key'] = $this->public_key;
					$simplepay_params['email'] = $email;
					$simplepay_params['address'] = $address;
					$simplepay_params['city'] = $city;
					$simplepay_params['country'] = $country;
					$simplepay_params['amount'] = $amount;
					$simplepay_params['order_id'] = $order_id;
					$simplepay_params['description'] = $description;
					$simplepay_params['currency'] = 'NGN';
					$simplepay_params['logo'] = $this->logo_url;
				}
			}

			wp_localize_script('mxmerchant', 'mxmerchant_params', $simplepay_params);
		}
		
		function add_error_ip_address($ip) {
			global $wpdb, $table_prefix;
			$wp_block = $table_prefix.'payment_request';
			$q = $wpdb->prepare("SELECT * FROM $wp_block WHERE requestIP = %s", $ip);
			$results = $wpdb->get_row($q);
			if(!empty($results)) {
				$updated_attempts = $results->attempt + 1;
				if($updated_attempts == 5) {
					$q = $wpdb->prepare("UPDATE $wp_block SET `attempt` = %d , `status` =  %s  WHERE requestIP = %s", $updated_attempts, "BLOCK", $ip);
					$wpdb->get_row($q);
				} else {
					$q = $wpdb->prepare("UPDATE $wp_block SET `attempt` = %d WHERE requestIP = %s", $updated_attempts, $ip);
					$wpdb->get_row($q);
				}
			} else {
				$data = array(
					'requestIP' => $ip,
					'time' => date("y-m-d h:i:s"),
					'attempt' => 1,
					'status' => 'OPEN'		
				);
				$wpdb->insert($wp_block, $data);
			}
		}

		function check_ip_address_before_payment($ip, $wc_order) {
			global $wpdb, $table_prefix;
			$wp_block = $table_prefix.'payment_request';
			$q = $wpdb->prepare("SELECT * FROM $wp_block WHERE requestIP = %s", $ip);
			$results = $wpdb->get_row($q);
			if(!empty($results)) {
				$hour_diff = round((strtotime(date("y-m-d h:i:s")) - strtotime($results->time))/3600, 1);
				if($hour_diff > 12) {
					$q = $wpdb->prepare("DELETE FROM $wp_block WHERE requestIP = %s", $ip);
					$wpdb->get_row($q);
				}
			}

			$q = $wpdb->prepare("SELECT * FROM $wp_block WHERE requestIP = %s", $ip);
			$results = $wpdb->get_row($q);
			if(!empty($results)) {
				if($results->status == 'BLOCK') {
					return "BLOCK";
				} 
			}
			return "ALLOW";
		}

		/**
		 * Process the payment and return the result
		 * */
		public function process_payment($order_id) {
			$enabled_ach_blocking = $this->get_option('enable_ip_blocking');
			$error_ach_message = $this->get_option('custom_error_message');
			$wc_order = wc_get_order($order_id);
			$check_block = $this->check_ip_address_before_payment($wc_order->customer_ip_address, $wc_order);
			if($check_block == 'BLOCK' && $enabled_ach_blocking == 'yes') {
				$wc_order->add_order_note(__($error_ach_message.', error on ' . date("d-M-Y h:i:s e"), 'woocommerce'));	 
				wc_add_notice($error_ach_message, 'error');
			} else {
				$route_no = sanitize_text_field(str_replace(' ', '', $_POST['mxmerchant_ach_gateway-routing-number']));
				$account_no = sanitize_text_field(str_replace(' ', '', $_POST['mxmerchant_ach_gateway-account-number']));


				$data = [
					'merchantId' => $this->merchant_id,
					'amount' => $wc_order->get_total(),
					'tax' => $wc_order->get_total_tax(),
					'shipAmount' => $wc_order->get_total_shipping(),
					'paymentType' => 'Sale',
					'tenderType' => 'Check',
					'bankAccount' => [
						'accountNumber' => $route_no,
						'routingNumber' => $account_no,
						'name' => $wc_order->get_billing_first_name() . ' ' . $wc_order->get_billing_last_name()
					],
					'customerName' => $wc_order->get_billing_first_name() . ' ' . $wc_order->get_billing_last_name(),
					'source' => 'API - WooCommerce',
					'meta' => get_bloginfo('blogname') . ' Order #'.$wc_order->get_order_number()
				];
				
				$result = $this->_api('checkout/v3/payment', $data);
				
				$json = json_decode($result);
				
				if ($json) {
					if ($json->status && $json->reference && $json->status === 'Approved') {
						$wc_order->add_order_note(__( $json->authMessage. ' on '.date("d-M-Y h:i:s e"). ' with Transaction ID = ' . $json->reference . ' using ' . strtoupper($json->tenderType) . ' and authorization code ' . $json->authCode, 'woocommerce'));
						$wc_order->payment_complete($json->reference);
						WC()->cart->empty_cart();
						return array (
							'result'   => 'success',
							'redirect' => $this->get_return_url( $wc_order ),
						);
					} else if ($json->errorCode && $json->details && isset($json->details[0])) {
						$this->add_error_ip_address($wc_order->customer_ip_address);
						$wc_order->add_order_note(__($json->details[0] . ' on ' . date("d-M-Y h:i:s e"), 'woocommerce'));	 
						wc_add_notice($json->details[0], 'error' );
					} else {
						$this->add_error_ip_address($wc_order->customer_ip_address);
						$wc_order->add_order_note(__('Unknown error on ' . date("d-M-Y h:i:s e"), 'woocommerce'));	 
						wc_add_notice('Unknown error', 'error');
					}
				} else if ($json->status) {
					$this->add_error_ip_address($wc_order->customer_ip_address);
					$wc_order->add_order_note(__($json->status . ' on ' . date("d-M-Y h:i:s e"), 'woocommerce'));	 
					wc_add_notice($json->status, 'error');
				} else {
					$this->add_error_ip_address($wc_order->customer_ip_address);
					$wc_order->add_order_note(__('Unknown error on ' . date("d-M-Y h:i:s e"), 'woocommerce'));	 
					wc_add_notice('Unknown error', 'error');
				}
			}
		}

		/**
		 * Output for the order received page.
		 * */
		public function receipt_page($order_id) {

			$order = wc_get_order($order_id);

			echo '<p>Thank you for your order, please click the button below to pay with debit/credit card using MxMerchant.</p>';

			echo '<div id="simplepay_form"><form id="order_review" method="post" action="' . WC()->api_request_url('MxMerchant_Gateway') . '"></form><button class="button alt" id="simplepay-payment-button">Pay Now</button> <a class="button cancel" href="' . esc_url($order->get_cancel_order_url()) . '">Cancel order &amp; restore cart</a></div>
			';
		}

		/**
		 * Verify a payment token
		 * */
		public function charge_token() {

			if (isset($_POST['mxmerchant_token'], $_POST['mxmerchant_order_id'])) {

				$verify_url = 'https://checkout.simplepay.ng/v2/payments/card/charge/';

				$order_id = (int) $_POST['mxmerchant_order_id'];

				$order = wc_get_order($order_id);
				$order_total = $order->get_total() * 100;

				$headers = array(
					'Content-Type' => 'application/json',
					'Authorization' => 'Basic ' . base64_encode($this->private_key . ':' . '')
				);

				$body = array(
					'token' => $_POST['mxmerchant_token'],
					'amount' => $order_total,
					'amount_currency' => 'NGN',
				);

				$args = array(
					'headers' => $headers,
					'body' => json_encode($body),
					'timeout' => 60,
					'method' => 'POST'
				);

				$request = wp_remote_post($verify_url, $args);

				if (!is_wp_error($request) && 200 == wp_remote_retrieve_response_code($request)) {

					$simplepay_response = json_decode(wp_remote_retrieve_body($request));

					$amount_paid = $simplepay_response->amount;
					$transaction_id = $simplepay_response->id;

					do_action('mxmerchant_after_payment', $simplepay_response);

					if ('20000' == $simplepay_response->response_code) {

						// check if the amount paid is equal to the order amount.
						if ($amount_paid < $order_total) {

							//Update the order status
							$order->update_status('on-hold', '');

							add_post_meta($order_id, '_transaction_id', $transaction_id, true);

							//Error Note
							$notice = 'Thank you for shopping with us.<br />The payment was successful, but the amount paid is not the same as the order amount.<br />Your order is currently on-hold.<br />Kindly contact us for more information regarding your order and payment status.';

							$notice_type = 'notice';

							//Add Admin Order Note
							$order->add_order_note('Look into this order. <br />This order is currently on hold.<br />Reason: Amount paid is less than the order amount.<br />Amount Paid was &#8358;' . $amount_paid / 100 . ' while the order amount is &#8358;' . $order_total / 100 . '<br />Simplepay Transaction ID: ' . $transaction_id);

							// Reduce stock levels
							$order->reduce_order_stock();

							wc_add_notice($notice, $notice_type);
						} else {

							$order->payment_complete($transaction_id);

							$order->add_order_note(sprintf('Payment via Simplepay successful (Transaction ID: %s)', $transaction_id));
						}

						wc_empty_cart();

						wp_redirect($this->get_return_url($order));

						exit;
					} else {

						wp_redirect(wc_get_page_permalink('checkout'));

						exit;
					}
				}
			}

			wp_redirect(wc_get_page_permalink('checkout'));

			exit;
		}

	}
	
	/**
	 * Add MxMerchant Gateway to WC
	 * */
	function tbz_wc_add_mxmerchant_gateway($methods) {
		$methods[] = 'MxMerchant_Gateway';
		// $methods[] = 'MxMerchant_ACH_Gateway';
		return $methods;
	}

	add_filter('woocommerce_payment_gateways', 'tbz_wc_add_mxmerchant_gateway');

	/**
	 * Add Settings link to the plugin entry in the plugins menu
	 * */
	function tbz_mxmerchant_plugin_action_links($links, $file) {

		static $this_plugin;

		if (!$this_plugin) {

			$this_plugin = plugin_basename(__FILE__);
		}

		if ($file == $this_plugin) {

			$settings_link = '<a href="' . get_bloginfo('wpurl') . '/wp-admin/admin.php?page=wc-settings&tab=checkout&section=mxmerchant_gateway">Settings</a>';
			array_unshift($links, $settings_link);
		}

		return $links;
	}

	add_filter('plugin_action_links', 'tbz_mxmerchant_plugin_action_links', 10, 2);

	
	
	/**
	 * Display the testmode notice
	 * */
	function mxmerchant_testmode_notice() {

		$simplepay_settings = get_option('woocommerce_mxmerchant_ach_gateway_settings');
		$simplepay_settings2 = get_option('woocommerce_mxmerchant_gateway_settings');

		if($simplepay_settings && $simplepay_settings2 && count($simplepay_settings) && count($simplepay_settings2)) {
			$testmode = $simplepay_settings['testmode'] === 'yes' ? true : false;
			$testmode2 = $simplepay_settings2['testmode'] === 'yes' ? true : false;
			$enabled = $simplepay_settings['enabled'] === 'yes' ? true : false;
			$enabled2 = $simplepay_settings2['enabled'] === 'yes' ? true : false;
	
			$public_test_key = $simplepay_settings['test_consumer_key'];
			$private_test_key = $simplepay_settings['test_secret_key'];
	
			$public_live_key = $simplepay_settings['prod_consumer_key'];
			$private_live_key = $simplepay_settings['prod_secret_key'];
	
			$public_key = $testmode ? $public_test_key : $public_live_key;
			$private_key = $testmode ? $private_test_key : $private_live_key;
			
			$public_test_key2 = $simplepay_settings2['test_consumer_key'];
			$private_test_key2 = $simplepay_settings2['test_secret_key'];
	
			$public_live_key2 = $simplepay_settings2['prod_consumer_key'];
			$private_live_key2 = $simplepay_settings2['prod_secret_key'];

			$public_key2 = $testmode2 ? $public_test_key2 : $public_live_key2;
			$private_key2 = $testmode2 ? $private_test_key2 : $private_live_key2;
	
			if ($enabled && $testmode) {
				?>
				<div class="update-nag">
					MxMerchant test mode is still enabled. Click <a href="<?php echo get_bloginfo('wpurl') ?>/wp-admin/admin.php?page=wc-settings&tab=checkout&section=mxmerchant_ach_gateway">here</a> to disable it when you want to start accepting live payment on your site.
				</div>
				<?php
			}
			
			if ($enabled2 && $testmode2) {
				?>
				<div class="update-nag">
					MxMerchant test mode is still enabled. Click <a href="<?php echo get_bloginfo('wpurl') ?>/wp-admin/admin.php?page=wc-settings&tab=checkout&section=mxmerchant_gateway">here</a> to disable it when you want to start accepting live payment on your site.
				</div>
				<?php
			}
	
			// Check required fields
			if ($enabled && ! ($public_key && $private_key)) {
				echo '<div class="error"><p>' . sprintf('Please enter your MxMerchant ACH API keys <a href="%s">here</a> to be able to use the MxMerchant WooCommerce plugin.', admin_url('admin.php?page=wc-settings&tab=checkout&section=mxmerchant_ach_gateway')) . '</p></div>';
			}
			if ($enabled2 && ! ($public_key2 && $private_key2)) {
				echo '<div class="error"><p>' . sprintf('Please enter your MxMerchant API keys <a href="%s">here</a> to be able to use the MxMerchant WooCommerce plugin.', admin_url('admin.php?page=wc-settings&tab=checkout&section=mxmerchant_gateway')) . '</p></div>';
			}
		}
	}

	add_action('admin_notices', 'mxmerchant_testmode_notice');
}

function mx_woocommerce_plugin_activation() {
	global $wpdb, $table_prefix;
	$wp_block = $table_prefix.'payment_request';

	$q = "CREATE TABLE IF NOT EXISTS `$wp_block` (`ID` INT NOT NULL AUTO_INCREMENT , `requestIP` VARCHAR(20) NOT NULL , `time` DATETIME(6) NOT NULL , `attempt` INT NOT NULL , `status` TEXT NOT NULL , PRIMARY KEY (`ID`)) ENGINE = InnoDB;";

	$wpdb->query($q);
}

register_activation_hook(__FILE__, 'mx_woocommerce_plugin_activation');

function block_user_fun() {
	include 'admin-table/user-table.php';
}

function display_block_user() {
	$obj = new MxMerchant_Gateway();
    $blockValue = $obj->get_blocking_value();
	$ach_obj = new MxMerchant_ACH_Gateway();
	$blockachValue = $ach_obj->get_ach_blocking_value();
	if($blockValue == 'yes' || $blockachValue == 'yes') {
		add_submenu_page('woocommerce','Blocked User','Blocked User', 'manage_options','block-user', 'block_user_fun'); 
	}
}

add_action('admin_menu', 'display_block_user');

function delete_block() {
	$post_id = $_POST["id"];
	
	global $wpdb, $table_prefix;
	$wp_block = $table_prefix.'payment_request';

	$q = "DELETE FROM $wp_block WHERE ID=$post_id";
	$wpdb->get_results($q);
	
    wp_die();
}
add_action( 'wp_ajax_delete_block', 'delete_block' );

function load_custom_script_in_plugin() {
	$js_file_path = plugins_url( 'assets/js/myjs.js', __FILE__ ); // Adjust the path to your actual JavaScript file.
    wp_enqueue_script( 'custom_script', $js_file_path, array( 'jquery' ), '1.0', true );
}
add_action( 'admin_enqueue_scripts', 'load_custom_script_in_plugin' );

function check_connections() {

}
add_action( 'wp_ajax_check_connections', 'check_connections' );

add_action('before_woocommerce_init', function() {
    if (class_exists(\Automattic\WooCommerce\Utilities\FeaturesUtil::class) &&
        method_exists(\Automattic\WooCommerce\Utilities\FeaturesUtil::class, 'declare_compatibility')) {
        \Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility('custom_order_tables', __FILE__, true);
    }
});

add_action( 'woocommerce_blocks_loaded', 'register_order_approval_payment_method_type' );

/**
 * Custom function to register a payment method type
 */
function register_order_approval_payment_method_type() {
   
    if ( ! class_exists( 'Automattic\WooCommerce\Blocks\Payments\Integrations\AbstractPaymentMethodType' ) ) {
        return;
    }

    
    require_once plugin_dir_path(__FILE__) . 'class-woocommerce-mxmerchant-block-checkout.php';

    add_action(
        'woocommerce_blocks_payment_method_type_registration',
        function( Automattic\WooCommerce\Blocks\Payments\PaymentMethodRegistry $payment_method_registry ) {
            // Register an instance of MxMerchant_Gateway_Blocks
            $payment_method_registry->register( new MxMerchant_Gateway_Blocks );
        }
    );
}