const { createElement, Fragment, useState } = wp.element;
const { useEffect } = window.wp.element;
const { __ } = wp.i18n;
const { applyFilters } = wp.hooks;
const { registerCheckoutFilters } = window.wc.blocksCheckout;
const modifyPlaceOrderButtonLabel = ( defaultValue, extensions ) => {
	return 'Process Payment';
};
registerCheckoutFilters( 'example-extension', {
	placeOrderButtonLabel: modifyPlaceOrderButtonLabel,
});

// Retrieve settings
const settings = window.wc.wcSettings.getSetting('mxmerchant_gateway_data', {});
const label = window.wp.htmlEntities.decodeEntities(settings.title) || __('Credit Card', 'mxmerchant_gateway');

// Function to create and return the content as a React element
const createCreditCardContent = ({ id, methodPoweredBy, eventRegistration, emitResponse  }) => {
  const { onPaymentSetup } = eventRegistration;

  useEffect(() => {
    const unsubscribe = onPaymentSetup(async () => {
      // Example of custom data and validation logic
      const card_number = document.getElementById(`${id}-card-number`).value;
      const card_expiry = document.getElementById(`${id}-card-expiry`).value;
      const card_cvc = document.getElementById(`${id}-card-cvc`).value;

      // Validation logic
      let errorMessage = '';

      if (!card_number) {
        errorMessage = 'Card number is required.';
      } else if (!card_expiry) {
        errorMessage = 'Expiration date is required.';
      } else if (!card_cvc) {
        errorMessage = 'Card code (CVC) is required.';
      } else {
        return {
          type: emitResponse.responseTypes.SUCCESS,
          meta: {
            paymentMethodData: {
              'mxmerchant_gateway-card-number': card_number,
              'mxmerchant_gateway-card-expiry': card_expiry,
              'mxmerchant_gateway-card-cvc': card_cvc
            },
          },
        };
    }
   
      return {
        type: emitResponse.responseTypes.ERROR,
        message: errorMessage
      };
    });


    return () => {
      unsubscribe();
    };
  }, [emitResponse.responseTypes.ERROR, emitResponse.responseTypes.SUCCESS,emitResponse.responseTypes.FAIL, onPaymentSetup, id]);

  const [cardNumber, setCardNumber] = useState('');
  const [expiryDate, setExpiryDate] = useState('');
  const [cvc, setCvc] = useState('');

  // Function to format card number
  const formatCardNumber = (input) => {
    let cleaned = input.replace(/\D/g, '');
    let formatted = cleaned.replace(/(\d{4})(\d{1,3})/, '$1 $2');
    formatted = formatted.replace(/(\d{4} \d{4})(\d{1,3})/, '$1 $2');
    formatted = formatted.replace(/(\d{4} \d{4} \d{4})(\d{1,3})/, '$1 $2');
    return formatted.slice(0, 19); // Limit to 19 characters (16 digits + 3 spaces)
  };

   // Format input value as MM / YY
   const formatExpiryDate = (value) => {
    // Remove any non-digit characters
    let cleaned = value.replace(/\D/g, '');

    // Extract month and year parts
    let month = cleaned.slice(0, 2);
    let year = cleaned.slice(2, 6); // Adjust year length for YYYY

    // Format as MM / YYYY
    if (year.length > 0) {
      year = ' ' + year;
    }
    return month + (month && year ? ' / ' : '') + year;
  };

    // Format the input to only allow numeric characters
    const handleCVCChange = (e) => {
      // Extract the numeric value from the input
      const newValue = e.target.value.replace(/\D/g, '');
      setCvc(newValue);
    };

    // Handle changes for card number
  const handleCardNumberChange = (e) => {
    const inputValue = e.target.value;
    setCardNumber(formatCardNumber(inputValue));
  };

    // Handle changes for expiry date
    const handleExpiryDateChange = (e) => {
      const inputValue = e.target.value;
      setExpiryDate(formatExpiryDate(inputValue));
    };


  const fields = applyFilters('woocommerce_credit_card_form_fields', {
        'card-number-field': createElement('p', { className: 'form-row form-row-wide' },
          createElement('label', { htmlFor: `${id}-card-number` },
            __('Card Number', 'woocommerce'),
            createElement('span', { className: 'required' }, '*')
          ),
          createElement('input', {
            id: `${id}-card-number`,
            className: 'input-text wc-credit-card-form-card-number',
            type: 'text',
            maxLength: '20',
            autoComplete: 'off',
            placeholder: '•••• •••• •••• ••••',
            value:cardNumber,
            onChange:handleCardNumberChange,
            name: `${id}-card-number`
          })
        ),
        'card-expiry-field': createElement('p', { className: 'form-row form-row-first' },
          createElement('label', { htmlFor: `${id}-card-expiry` },
            __('Expiry (MM/YY)', 'woocommerce'),
            createElement('span', { className: 'required' }, '*')
          ),
          createElement('input', {
            id: `${id}-card-expiry`,
            className: 'input-text wc-credit-card-form-card-expiry',
            type: 'text',
            autoComplete: 'off',
            placeholder: __('MM / YY', 'woocommerce'),
            value:expiryDate,
            maxLength:10,
            onChange:handleExpiryDateChange,
            name: `${id}-card-expiry`
          })
        ),
        'card-cvc-field': createElement('p', { className: 'form-row form-row-last' },
          createElement('label', { htmlFor: `${id}-card-cvc` },
            __('Card Code', 'woocommerce'),
            createElement('span', { className: 'required' }, '*')
          ),
          createElement('input', {
            id: `${id}-card-cvc`,
            className: 'input-text wc-credit-card-form-card-cvc',
            type: 'text',
            autoComplete: 'off',
            placeholder: __('CVC', 'woocommerce'),
            name: `${id}-card-cvc`,
            value:cvc,
            maxLength:4,
            onChange:handleCVCChange,
          })
        ),
      }
    , id);
  
  const formContent = createElement('fieldset', { id: `wc-${id}-cc-form`, className: 'wc-credit-card-form wc-payment-form' },
    createElement(Fragment, null,
        wp.hooks.applyFilters('woocommerce_credit_card_form_start', null, id),
        Object.values(fields),
        wp.hooks.applyFilters('woocommerce_credit_card_form_end', null, id)
      ),
    createElement('div', { className: 'clear' }),
    createElement('p', { className: 'powered_by_text' }, methodPoweredBy)
  );

  return formContent;
  };
// Define the Credit_Card_Payment_Gateway  object
const Credit_Card_Payment_Gateway  = {
    name: 'mxmerchant_gateway',
    label: label,
    content: createElement(createCreditCardContent, { id: settings.id, methodPoweredBy: settings.method_powered_by }),
    edit: createElement(createCreditCardContent, { id: settings.id, methodPoweredBy: settings.method_powered_by }),
    canMakePayment: () => true,
    ariaLabel: label,
    save: (props) => {
      return `<div ${props}>My Custom Payment Gateway</div>`;
  },
    supports: {
        features: settings.supports,
    },
};

// Register the payment methods
window.wc.wcBlocksRegistry.registerPaymentMethod(Credit_Card_Payment_Gateway);
